<?php
/**
 * OSB V2-Payment Module version 1.5.0 for WooCommerce 2.x-3.x. Support contact : support@osb.pf.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * @author    Lyra Network (http://www.lyra-network.com/)
 * @author    Alsacréations (Geoffrey Crofte http://alsacreations.fr/a-propos#geoffrey)
 * @copyright 2014-2018 Lyra Network and contributors
 * @license   http://www.gnu.org/licenses/old-licenses/gpl-2.0.html  GNU General Public License (GPL v2)
 * @category  payment
 * @package   osb
 */

if (! defined('ABSPATH')) {
    exit; // exit if accessed directly
}

/**
 * OSB Payment Gateway : common class.
 */
class WC_Gateway_Osb extends WC_Payment_Gateway
{

    private static $success_order_statues = array(
        'on-hold' => 'On Hold',
        'processing' => 'Processing',
        'completed' => 'Complete'
    );

    protected $general_settings = array();
    protected $general_form_fields = array();

    public function __construct()
    {
        $this->id = 'osb';
        $this->has_fields = false;
        $this->method_title = 'OSB - ' . __('General configuration', 'woo-osb-payment');

        // init OSB common vars
        $this->osb_init();

        // load the form fields
        $this->init_form_fields();

        // load the module settings
        $this->init_settings();

        $this->title = __('General configuration', 'woo-osb-payment');
        $this->enabled = false;
        $this->testmode = ($this->get_general_option('ctx_mode') == 'TEST');
        $this->debug = ($this->get_general_option('debug') == 'yes') ? true : false;

        // reset osb common admin form action
        add_action('woocommerce_settings_start', array($this, 'osb_reset_admin_options'));

        // adding style to admin form action
        add_action('admin_head-woocommerce_page_' . $this->admin_page, array($this, 'osb_admin_head_style'));

        // update osb admin form action
        add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));

        // return from payment platform action
        add_action('woocommerce_api_wc_gateway_osb', array($this, 'osb_notify_response'));

        // filter to allow order status override
        add_filter('woocommerce_payment_complete_order_status', array($this, 'osb_complete_order_status'), 10, 2);

        // customize email
        add_action('woocommerce_email_after_order_table', array($this, 'osb_add_order_email_payment_result'), 10, 3);
    }

    protected function osb_init()
    {
        global $woocommerce;

        $this->logger = new WC_Logger();

        // init OSB API
        $this->osb_request = new OsbRequest();

        $this->admin_section = 'wc_gateway_' . $this->id;
        $this->admin_tab = 'checkout';
        $this->admin_page = 'wc-settings';

        // backward compatibility
        if (version_compare($woocommerce->version, '2.1.0', '<')) {
            $this->admin_section = get_class($this);
            $this->admin_tab = 'payment_gateways';
            $this->admin_page = 'woocommerce_settings';
        }

        // admin settings page URL
        $this->admin_link = add_query_arg(
            'section',
            $this->admin_section,
            add_query_arg('tab', $this->admin_tab, add_query_arg('page', $this->admin_page, admin_url('admin.php')))
        );

        // reset admin settings URL
        $this->reset_admin_link = $this->admin_link;
        $this->reset_admin_link = add_query_arg('noheader', 'true', add_query_arg('reset', 'true', $this->reset_admin_link));
        $this->reset_admin_link = wp_nonce_url($this->reset_admin_link, $this->admin_page);
    }

    public function osb_admin_head_style()
    {
        ?>
        <style>
            .osb p.description {
                color: #0073aa !important;
                font-style: normal !important;
            }

            #woocommerce_osb_url_check + p.description span {
                color: #23282d !important;
                font-size: 16px;
                font-weight: bold;
            }

            #woocommerce_osb_url_check + p.description img {
                vertical-align: middle;
                margin-right: 5px;
            }
        </style>
        <?php
    }

    /**
     * Admin Panel Options.
     */
    public function admin_options()
    {
        if (! $this->is_supported_currency()) {
            echo '<div class="inline error"><p><strong>' . __('Platform disabled', 'woo-osb-payment') . ': ' . sprintf(__('%s does not support your store currency.', 'woo-osb-payment'), 'OSB') . '</strong></p></div>';
        }

        if (get_transient($this->id . '_settings_reset')) {
            delete_transient($this->id . '_settings_reset');

            echo '<div class="inline updated"><p><strong>' . sprintf(__('Your %s module configuration is successfully reset.', 'woo-osb-payment'), 'OSB') . '</strong></p></div>';
        }
        ?>

        <br />
        <h3>OSB</h3>
        <p><?php echo sprintf(__('The module works by sending users to %s in order to select their payment mean and enter their payment information.', 'woo-osb-payment'), 'OSB'); ?></p>

        <section class="osb">
        <table class="form-table">
            <?php $this->generate_settings_html(); // generate the HTML For the settings form ?>
        </table>
        </section>

        <a href="<?php echo $this->reset_admin_link; ?>"><?php _e('Reset configuration', 'woo-osb-payment');?></a>

        <?php
    }

    public function osb_reset_admin_options()
    {
        // if not reset action do nothing
        if (! isset($_GET['reset'])) {
            return;
        }

        // check if correct link
        if ($this->admin_section != $_GET['section']) {
            return;
        }

        delete_option('woocommerce_' . $this->id . '_settings');

        // transcient flag to display reset message
        set_transient($this->id . '_settings_reset', 'true');

        wp_redirect($this->admin_link);
        die();
    }

    protected function get_supported_languages()
    {
        $langs = array();

        foreach (OsbApi::getSupportedLanguages() as $code => $label) {
            $langs[$code] = __($label, 'woo-osb-payment');
        }

        return $langs;
    }

    /**
     * Initialise Gateway Settings Form Fields
     */
    public function init_form_fields()
    {
        global $woocommerce, $osb_plugin_features;

        // get log folder
        if (function_exists('wc_get_log_file_path')) {
            $log_folder = dirname(wc_get_log_file_path('osb')) . '/';
        } else {
            $log_folder = $woocommerce->plugin_path() . '/logs/';
        }
        $log_folder = str_replace('\\', '/', $log_folder);

        // get relative path
        $base_dir = str_replace('\\', '/', ABSPATH);
        if (strpos($log_folder, $base_dir) === 0) {
            $log_folder = str_replace($base_dir, '', $log_folder);
        } else {
            $base_dir = str_replace('\\', '/', dirname(ABSPATH));
            $log_folder = str_replace($base_dir, '..', $log_folder);
        }

        // get documentation links
        $docs = '';
        $filenames = glob(plugin_dir_path(__FILE__) . 'installation_doc/OSB_WooCommerce_2.x-3.x_v1.5.0*.pdf');

        $languages = array(
            'fr' => 'Français',
            'en' => 'English',
            'es' => 'Español'
            // complete when other languages are managed
        );

        foreach ($filenames as $filename) {
            $base_filename = basename($filename, '.pdf');
            $lang = substr($base_filename, -2); // extract language code

            $docs .= '<a style="margin-left: 10px; text-decoration: none;" href="' . WC_OSB_PLUGIN_URL
                . 'installation_doc/' . $base_filename . '.pdf" target="_blank">[' . $languages[$lang] . ']</a>';
        }

        // prepare succes order statuses array
        $statues = array('default' => __('Default', 'woo-osb-payment'));
        foreach (self::$success_order_statues as $key => $value) {
            $statues[$key] = __($value, 'woo-osb-payment');
        }

        $this->form_fields = array(
                // module information
            'module_details' => array(
                'title' => __('MODULE DETAILS', 'woo-osb-payment'),
                'type' => 'title'
            ),

            'developped_by' => array(
                'title' => __('Developed by', 'woo-osb-payment'),
                'type' => 'text',
                'description' => '<b><a href="http://www.lyra-network.com/" target="_blank">Lyra Network</a></b>',
                'css' => 'display: none;'
            ),
            'contact' => array(
                'title' => __('Contact us', 'woo-osb-payment'),
                'type' => 'text',
                'description' => '<b><a href="mailto:support@osb.pf">support@osb.pf</a></b>',
                'css' => 'display: none;'
            ),
            'contrib_version' => array(
                'title' => __('Module version', 'woo-osb-payment'),
                'type' => 'text',
                'description' => '1.5.0',
                'css' => 'display: none;'
            ),
            'platform_version' => array(
                'title' => __('Platform version', 'woo-osb-payment'),
                'type' => 'text',
                'description' => 'V2',
                'css' => 'display: none;'
            ),
            'doc_link' => array(
                'title' => __('Click to view the module configuration documentation :', 'woo-osb-payment') . $docs,
                'type' => 'label',
                'css' => 'font-weight: bold; color: red; cursor: auto !important;'
            ),

            'base_settings' => array(
                'title' => __('BASE SETTINGS', 'woo-osb-payment'),
                'type' => 'title'
            ),
            'debug' => array(
                'title' => __('Logs', 'woo-osb-payment'),
                'label' => __('Enable / disable', 'woo-osb-payment'),
                'type' => 'checkbox',
                'default' => 'yes',
                'description' => sprintf(__('Enable / disable module logs. The log file will be inside <code>%s</code>.', 'woo-osb-payment'), $log_folder),
            ),

            // payment platform access params
            'payment_platform_access' => array(
                'title' => __('PAYMENT PLATFORM ACCESS', 'woo-osb-payment'),
                'type' => 'title'
            ),
            'site_id' => array(
                'title' => __('Shop ID', 'woo-osb-payment'),
                'type' => 'text',
                'default' => '12345678',
                'description' => sprintf(__('The identifier provided by %s.', 'woo-osb-payment'), 'OSB'),
                'custom_attributes' => array('autocomplete' => 'off')
            ),
            'key_test' => array(
                'title' => __('Certificate in test mode', 'woo-osb-payment'),
                'type' => 'text',
                'default' => '1111111111111111',
                'description' => sprintf(__('Certificate provided by %s for test mode (available in %s Back Office).', 'woo-osb-payment'), 'OSB', 'OSB'),
                'custom_attributes' => array('autocomplete' => 'off')
            ),
            'key_prod' => array(
                'title' => __('Certificate in production mode', 'woo-osb-payment'),
                'type' => 'text',
                'default' => '2222222222222222',
                'description' => sprintf(__('Certificate provided by %s (available in %s Back Office after enabling production mode).', 'woo-osb-payment'), 'OSB', 'OSB'),
                'custom_attributes' => array('autocomplete' => 'off')
            ),
            'ctx_mode' => array(
                'title' => __('Mode', 'woo-osb-payment'),
                'type' => 'select',
                'default' => 'TEST',
                'options' => array(
                    'TEST' => __('TEST', 'woo-osb-payment'),
                    'PRODUCTION' => __('PRODUCTION', 'woo-osb-payment')
                ),
                'description' => __('The context mode of this module.', 'woo-osb-payment'),
                'class' => 'wc-enhanced-select'
            ),
            /*'sign_algo' => array(
                'title' => __('Signature algorithm', 'woo-osb-payment'),
                'type' => 'select',
                'default' => 'SHA-1',
                'options' => array(
                    OsbApi::ALGO_SHA1 => OsbApi::ALGO_SHA1,
                    OsbApi::ALGO_SHA256 => OsbApi::ALGO_SHA256
                ),
                'description' => sprintf(__('Algorithm used to compute the payment form signature. <b>Selected algorithm must be the same as one configured in the %s Back Office for the current context mode.</b>', 'woo-osb-payment'),'OSB'),
                'class' => 'wc-enhanced-select'
            ),*/
            'url_check' => array(
                'title' => __('Instant Payment Notification URL', 'woo-osb-payment'),
                'type' => 'text',
                'description' => '<span>' . add_query_arg('wc-api', 'WC_Gateway_Osb', network_home_url('/')) . '</span><br />' .
                    '<img src="' . esc_url(WC_OSB_PLUGIN_URL . 'assets/images/warn.png') . '">' . sprintf(__('URL to copy into your %s Back Office > Settings > Notification rules.', 'woo-osb-payment'), 'OSB'),
                'css' => 'display: none;'
            ),
            'platform_url' => array(
                'title' => __('Payment page URL', 'woo-osb-payment'),
                'type' => 'text',
                'default' => 'https://secure.osb.pf/vads-payment/',
                'description' => __('Link to the payment page.', 'woo-osb-payment'),
                'css' => 'width: 350px;'
            ),

            // payment page params
            'payment_page' => array(
                'title' => __('PAYMENT PAGE', 'woo-osb-payment'),
                'type' => 'title'
            ),
            'language' => array(
                'title' => __('Default language', 'woo-osb-payment'),
                'type' => 'select',
                'default' => 'fr',
                'options' => $this->get_supported_languages(),
                'description' => __('Default language on the payment page.', 'woo-osb-payment'),
                'class' => 'wc-enhanced-select'
            ),
            'available_languages' => array(
                'title' => __('Available languages', 'woo-osb-payment'),
                'type' => 'multiselect',
                'default' => array(),
                'options' => $this->get_supported_languages(),
                'description' => __('Languages available on the payment page. If you do not select any, all the supported languages will be available.', 'woo-osb-payment'),
                'class' => 'wc-enhanced-select'
            ),
            'capture_delay' => array(
                'title' => __('Capture delay', 'woo-osb-payment'),
                'type' => 'text',
                'default' => '',
                'description' => sprintf(__('The number of days before the bank capture (adjustable in your %s Back Office).', 'woo-osb-payment'), 'OSB')
            ),
            'validation_mode' => array(
                'title' => __('Validation mode', 'woo-osb-payment'),
                'type' => 'select',
                'default' => '',
                'options' => array(
                    '' => __('Back Office configuration', 'woo-osb-payment'),
                    '0' => __('Automatic', 'woo-osb-payment'),
                    '1' => __('Manual', 'woo-osb-payment')
                ),
                'description' => sprintf(__('If manual is selected, you will have to confirm payments manually in your %s Back Office.', 'woo-osb-payment'), 'OSB'),
                'class' => 'wc-enhanced-select'
            ),

            // selective 3DS
            'selective_3ds' => array(
                'title' => __('SELECTIVE 3DS', 'woo-osb-payment'),
                'type' => 'title'
            ),
            '3ds_min_amount' => array(
                'title' => __('Minimum amount to activate 3-DS', 'woo-osb-payment'),
                'type' => 'text',
                'default' => '',
                'description' => __('Needs subscription to Selective 3-D Secure option.', 'woo-osb-payment')
            ),

            // return to store params
            'return_options' => array(
                'title' => __('RETURN OPTIONS', 'woo-osb-payment'),
                'type' => 'title'
            ),
            'redirect_enabled' => array(
                'title' => __('Automatic redirection', 'woo-osb-payment'),
                'label' => __('Enable / disable', 'woo-osb-payment'),
                'type' => 'checkbox',
                'default' => 'no',
                 'description' => __('If enabled, the buyer is automatically redirected to your site at the end of the payment.', 'woo-osb-payment')
            ),
            'redirect_success_timeout' => array(
                'title' => __('Redirection timeout on success', 'woo-osb-payment'),
                'type' => 'text',
                'default' => '5',
                'description' => __('Time in seconds (0-300) before the buyer is automatically redirected to your website after a successful payment.', 'woo-osb-payment')
            ),
            'redirect_success_message' => array(
                'title' => __('Redirection message on success', 'woo-osb-payment'),
                'type' => 'multilangtext',
                'default' => array(
                    'en_US' => 'Redirection to shop in a few seconds...',
                    'en_GB' => 'Redirection to shop in a few seconds...',
                    'fr_FR' => 'Redirection vers la boutique dans quelques instants...',
                    'de_DE' => 'Weiterleitung zum Shop in Kürze...'
                ),
                'description' => __('Message displayed on the payment page prior to redirection after a successful payment.', 'woo-osb-payment'),
                'css' => 'width: 35em;'
            ),
            'redirect_error_timeout' => array(
                'title' => __('Redirection timeout on failure', 'woo-osb-payment'),
                'type' => 'text',
                'default' => '5',
                'description' => __('Time in seconds (0-300) before the buyer is automatically redirected to your website after a declined payment.', 'woo-osb-payment')
            ),
            'redirect_error_message' => array(
                'title' => __('Redirection message on failure', 'woo-osb-payment'),
                'type' => 'multilangtext',
                'default' => array(
                    'en_US' => 'Redirection to shop in a few seconds...',
                    'en_GB' => 'Redirection to shop in a few seconds...',
                    'fr_FR' => 'Redirection vers la boutique dans quelques instants...',
                    'de_DE' => 'Weiterleitung zum Shop in Kürze...'
                ),
                'description' => __('Message displayed on the payment page prior to redirection after a declined payment.', 'woo-osb-payment'),
                'css' => 'width: 35em;'
            ),
            'return_mode' => array(
                'title' => __('Return mode', 'woo-osb-payment'),
                'type' => 'select',
                'default' => 'GET',
                'options' => array(
                    'GET' => 'GET',
                    'POST' => 'POST'
                ),
                'description' => __('Method that will be used for transmitting the payment result from the payment page to your shop.', 'woo-osb-payment'),
                'class' => 'wc-enhanced-select'
            ),
            'order_status_on_success' => array(
                'title' => __('Order Status', 'woo-osb-payment'),
                'type' => 'select',
                'default' => 'default',
                'options' => $statues,
                'description' => __('Defines the status of orders paid with this payment mode.', 'woo-osb-payment'),
                'class' => 'wc-enhanced-select'
            )
        );

        if ($osb_plugin_features['qualif']) {
            // tests will be made on qualif, no test mode available
            unset($this->form_fields['key_test']);

            $this->form_fields['ctx_mode']['disabled'] = true;
        }

        // save general form fields
        foreach ($this->form_fields as $k => $v) {
            $this->general_form_fields[$k] = $v;
        }
    }

    protected function init_general_settings()
    {
        $this->general_settings = get_option('woocommerce_osb_settings', null);

        // if there are no settings defined, use defaults
        if (! is_array($this->general_settings) || empty($this->general_settings)) {
            $this->general_settings = array();

            foreach ($this->general_form_fields as $k => $v) {
                $this->general_settings[$k] = isset($v['default']) ? $v['default'] : '';
            }
        }
    }

    protected function get_general_option($key, $empty_value = null)
    {
        if (empty($this->general_settings)) {
            $this->init_general_settings();
        }

        // get empty string if unset
        if (! isset($this->general_settings[$key])) {
            $this->general_settings[$key] = '';
        }

        if (! is_null($empty_value) && ($this->general_settings[$key] === '')) {
            $this->general_settings[$key] = $empty_value;
        }

        return $this->general_settings[$key];
    }

    public function generate_label_html($key, $data)
    {
        $defaults = array(
            'title'             => '',
            'class'             => '',
            'css'               => '',
            'placeholder'       => '',
            'type'              => 'label',
            'description'       => ''
        );

        $data = wp_parse_args($data, $defaults);

        ob_start();
        if ($data != null) {
        ?>
            <tr valign="top">
                <td class="forminp" colspan="2" style="padding-left: 0;">
                    <fieldset>
                        <label class="<?php echo esc_attr($data['class']); ?>" style="<?php echo esc_attr($data['css']); ?>"><?php echo wp_kses_post($data['title']); ?></label>
                        <p class="description"><?php echo wp_kses_post($data['description']); ?></p>
                    </fieldset>
                </td>
            </tr>
        <?php
        }

        return ob_get_clean();
    }

    public function generate_multilangtext_html($key, $data)
    {
        global $wp_version;

        $data['title']           = isset($data['title']) ? $data['title'] : '';
        $data['disabled']        = empty($data['disabled']) ? false : true;
        $data['class']           = isset($data['class']) ? ' ' . $data['class'] : '';
        $data['css']             = isset($data['css']) ? $data['css'] : '';
        $data['placeholder']     = isset($data['placeholder']) ? $data['placeholder'] : '';
        $data['type']            = isset($data['type']) ? $data['type'] : 'array';
        $data['desc_tip']        = isset($data['desc_tip']) ? $data['desc_tip'] : false;
        $data['description']     = isset($data['description']) ? $data['description'] : '';
        $data['default']         = isset($data['default']) ? $data['default'] : array('en_US' => '');

        $languages = get_available_languages();
        foreach ($languages as $lang) {
            if (! isset($data['default'][$lang])) {
                $data['default'][$lang] = $data['default']['en_US'];
            }
        }
        $field = $this->plugin_id . $this->id . '_' . $key;
        $value = $this->get_option($key);

        // set input default value
        $default_input_value = isset($value[get_locale()]) ? $value[get_locale()] : $data['default'][get_locale()];
        $default_input_value = esc_attr($default_input_value);

        ob_start();
        ?>

        <tr valign="top">
            <th scope="row" class="titledesc">
                <label for="<?php echo esc_attr($field); ?>_text"><?php echo wp_kses_post($data['title']); ?></label>
            </th>
            <td class="forminp">
                <fieldset>
                    <legend class="screen-reader-text"><span><?php echo wp_kses_post($data['title']); ?></span></legend>
                    <input class="input-text regular-input<?php echo esc_attr($data['class']); ?>" type="text"
                            name="<?php echo esc_attr($field) . '[text]'; ?>" id="<?php echo esc_attr($field) . '_text'; ?>" style="<?php echo esc_attr($data['css']); ?>"
                            value="<?php echo $default_input_value; ?>" placeholder="<?php echo esc_attr($data['placeholder']); ?>" <?php disabled($data['disabled'], true); ?>>

                    <?php
                    if (version_compare($wp_version, '4.0.0', '>=')) {
                        $select = wp_dropdown_languages(array(
                            'name'         => esc_attr($field) . '[lang]',
                            'id'           => esc_attr($field) . '_lang',
                            'selected'     => get_locale(), // default selected is current admin locale
                            'languages'    => $languages,
                            'translations' => array(),
                            'show_available_translations' => false,
                            'echo' => false
                        ));

                        echo str_replace('<select', '<select style="width: auto; height: auto; vertical-align: top;"', $select);
                    } else {
                        $languages = array();
                    }

                    $languages[] = 'en_US';
                    foreach ($languages as $lang) {
                        $v = isset($value[$lang]) ? $value[$lang] : $data['default'][$lang]; ?>
                        <input type="hidden" id="<?php echo esc_attr($field) . '_' . $lang; ?>"
                                name="<?php echo esc_attr($field) . '[' .  $lang . ']'; ?>"
                                value="<?php echo esc_attr($v); ?>">
                    <?php
                    }
                    ?>

                    <p class="description"><?php echo wp_kses_post($data['description']); ?></p>
                </fieldset>
            </td>
        </tr>

        <script type="text/javascript">
            jQuery(document).ready(function() {
                var key = '#<?php echo esc_attr($field); ?>';
                jQuery(key + '_lang').bind('change', function() {
                    var sl = jQuery(key + '_lang').val() || 'en_US';
                    var value = jQuery(key + '_' + sl).val();
                    jQuery(key + '_text').val(value);
                });

                jQuery(key + '_text').bind('change', function() {
                    var sl = jQuery(key + '_lang').val() || 'en_US';
                    var value = jQuery(key + '_text').val();
                    jQuery(key + '_' + sl).val(value);
                });
            });
        </script>
        <?php

        return ob_get_clean();
    }

    public function validate_multilangtext_field($key, $value = null)
    {
        $name = $this->plugin_id . $this->id . '_' . $key;
        $old_value = $this->get_option($key);
        $new_value = ! is_null($value) ? $value : (key_exists($name, $_POST) ? $_POST[$name] : '');

        if (isset($new_value) && is_array($new_value) && ! empty($new_value)) {
            unset($new_value['text']);
            unset($new_value['lang']);

            $languages = get_available_languages();
            $languages[] = 'en_US'; // en_US locale is always available for WP
            foreach ($languages as $lang) {
                if (! isset($new_value[$lang]) || ! $new_value[$lang]) {
                    $new_value[$lang] = $old_value[$lang];
                }
            }

            return $new_value;
        } else {
            return $old_value;
        }
    }

    /**
     * Validate multiselect field.
     *
     * @return array
     */
    public function validate_multiselect_field($key, $value = null)
    {
        $name = $this->plugin_id . $this->id . '_' . $key;
        $new_value = ! is_null($value) ? $value : (key_exists($name, $_POST) ? $_POST[$name] : array(''));

        if (isset($new_value) && is_array($new_value) && in_array('', $new_value)) {
            return array('');
        } else {
            return parent::validate_multiselect_field($key, $value);
        }
    }

    public function validate_ctx_mode_field($key, $value = null)
    {
        global $osb_plugin_features;

        $name = $this->plugin_id . $this->id . '_' . $key;
        $new_value = ! is_null($value) ? $value : (key_exists($name, $_POST) ? $_POST[$name] : null);

        if (! $new_value && $osb_plugin_features['qualif']) {
            // when using qualif for testing, mode is always PRODUCTION
            return 'PRODUCTION';
        }

        return parent::validate_select_field($key, $value);
    }

    /**
     * Check if this gateway is available for the current currency.
     */
    protected function is_supported_currency()
    {
        $currency = OsbApi::findCurrencyByAlphaCode(get_woocommerce_currency());
        if ($currency == null) {
            return false;
        }

        return true;
    }

    /**
     * Check for OSB notify Response.
     **/
    public function osb_notify_response()
    {
        global $woocommerce;

        @ob_clean();

        $raw_response = (array) stripslashes_deep($_REQUEST);

        $osb_response = new OsbResponse(
            $raw_response,
            $this->get_general_option('ctx_mode'),
            $this->get_general_option('key_test'),
            $this->get_general_option('key_prod'),
            $this->get_general_option('sign_algo')
        );

        $from_server = $osb_response->get('hash') != null;

        if (! $osb_response->isAuthentified()) {
            $this->log('Authentication failed: received invalid response with parameters: ' . print_r($raw_response, true));
            // $this->log('Signature algorithm selected in module settings must be the same as one selected in OSB Back Office.');

            if ($from_server) {
                $this->log('IPN URL PROCESS END');
                die($osb_response->getOutputForPlatform('auth_fail'));
            } else {
                // fatal error, empty cart
                $woocommerce->cart->empty_cart();
                $this->add_notice(__('An error has occured in the payment process.', 'woo-osb-payment'), 'error');

                $this->log('RETURN URL PROCESS END');

                $cart_url = function_exists('wc_get_cart_url') ? wc_get_cart_url() : $woocommerce->cart->get_cart_url();
                $iframe = $osb_response->get('action_mode') === 'IFRAME';
                $this->osb_redirect($cart_url, $iframe);
            }
        } else {
            header('HTTP/1.1 200 OK');

            $this->osb_manage_notify_response($osb_response);
        }
    }

    /**
     * Valid payment process : update order, send mail, ...
     **/
    public function osb_manage_notify_response($osb_response)
    {
        global $woocommerce, $osb_plugin_features;

        // clear all response messages
        $this->clear_notices();

        $order_id = $osb_response->get('order_id');
        $from_server = $osb_response->get('hash') != null;
        $iframe = $osb_response->get('action_mode') == 'IFRAME';

        // cart URL
        $cart_url = function_exists('wc_get_cart_url') ? wc_get_cart_url() : $woocommerce->cart->get_cart_url();

        $order = new WC_Order((int) $order_id);
        if (! $this->get_order_property($order, 'id') || ($this->get_order_property($order, 'order_key') !== $osb_response->get('order_info'))) {
            $this->log("Error: order #$order_id not found or key does not match received invoice ID.");

            if ($from_server) {
                $this->log('IPN URL PROCESS END');
                die($osb_response->getOutputForPlatform('order_not_found'));
            } else {
                // fatal error, empty cart
                $woocommerce->cart->empty_cart();
                $this->add_notice(__('An error has occured in the payment process.', 'woo-osb-payment'), 'error');

                $this->log('RETURN URL PROCESS END');
                $this->osb_redirect($cart_url, $iframe);
            }
        }

        if ($this->testmode && $osb_plugin_features['prodfaq']) {
            $msg = __('<p><u>GOING INTO PRODUCTION</u></p>You want to know how to put your shop into production mode, please go to this URL: ', 'woo-osb-payment');
            $msg .= '<a href="https://secure.osb.pf/html/faq/prod" target="_blank">https://secure.osb.pf/html/faq/prod</a>';

            $this->add_notice($msg);
        }

        // checkout payment URL to allow re-order
        $error_url = function_exists('wc_get_checkout_url') ? wc_get_checkout_url() : $woocommerce->cart->get_checkout_url();

        // backward compatibility
        if (version_compare($woocommerce->version, '2.1.0', '<')) {
            $error_url = $order->get_cancel_order_url();
        }

        if ($this->is_new_order($order, $osb_response->get('trans_id'))) {
            // order not processed yet or a failed payment (re-order)

            // delete old saved transaction details
            delete_post_meta((int) $order_id, 'Transaction ID');
            delete_post_meta((int) $order_id, 'Card number');
            delete_post_meta((int) $order_id, 'Payment mean');
            delete_post_meta((int) $order_id, 'Card expiry');

            // store transaction details
            update_post_meta((int) $order_id, 'Transaction ID', $osb_response->get('trans_id'));
            update_post_meta((int) $order_id, 'Card number', $osb_response->get('card_number'));
            update_post_meta((int) $order_id, 'Payment mean', $osb_response->get('card_brand'));

            $expiry = '';
            if ($osb_response->get('expiry_month') && $osb_response->get('expiry_year')) {
                $expiry = str_pad($osb_response->get('expiry_month'), 2, '0', STR_PAD_LEFT) . '/' . $osb_response->get('expiry_year');
            }
            update_post_meta((int) $order_id, 'Card expiry', $expiry);

            // Add order note
            $this->osb_add_order_note($osb_response, $order);

            if ($osb_response->isAcceptedPayment()) {
                if ($osb_response->isPendingPayment()) {
                    // payment is pending
                    $this->log("Payment is pending, make order #$order_id on-hold (pending).");

                    $order->update_status('on-hold');
                } else {
                    // payment completed
                    $this->log("Payment successfull, let's save order #$order_id.");

                    $order->payment_complete();
                }

                if ($from_server) {
                    $this->log("Payment processed successfully by IPN URL call for order #$order_id.");
                    $this->log('IPN URL PROCESS END');

                    die($osb_response->getOutputForPlatform('payment_ok'));
                } else {
                    $this->log("Warning ! IPN URL call has not worked. Payment completed by return URL call for order #$order_id.");

                    if ($this->testmode) {
                        $ipn_url_warn = sprintf(__('The automatic notification (peer to peer connection between the payment platform and your shopping cart solution) hasn\'t worked. Have you correctly set up the notification URL in the %s Back Office ?', 'woo-osb-payment'), 'OSB');
                        $ipn_url_warn .= '<br />';
                        $ipn_url_warn .= __('For understanding the problem, please read the documentation of the module : <br />&nbsp;&nbsp;&nbsp;- Chapter &laquo;To read carefully before going further&raquo;<br />&nbsp;&nbsp;&nbsp;- Chapter &laquo;Notification URL settings&raquo;', 'woo-osb-payment');

                        $this->add_notice($ipn_url_warn, 'error');
                    }

                    $this->log('RETURN URL PROCESS END');
                    $this->osb_redirect($this->get_return_url($order), $iframe);
                }
            } else {
                $order->update_status('failed');
                $this->log("Payment failed or cancelled for order #$order_id. {$osb_response->getLogString()}");

                if ($from_server) {
                    $this->log('IPN URL PROCESS END');
                    die($osb_response->getOutputForPlatform('payment_ko'));
                } else {
                    if (! $osb_response->isCancelledPayment()) {
                        $this->add_notice(__('Your payment was not accepted. Please, try to re-order.', 'woo-osb-payment'), 'error');
                    }

                    $this->log('RETURN URL PROCESS END');
                    $this->osb_redirect($error_url, $iframe);
                }
            }
        } else {
            $this->log("Order #$order_id is already saved. Update status or just show result according to case.");

            if ($from_server && ($this->get_order_property($order, 'status') === 'on-hold')) {
                switch (true) {
                    case $osb_response->isPendingPayment():
                        $this->log("Order #$order_id is in a pending status and stay in the same status.");
                        echo($osb_response->getOutputForPlatform('payment_ok_already_done'));
                        break;
                    case $osb_response->isAcceptedPayment():
                        $this->log("Order #$order_id is in a pending status and payment is accepted. Complete order payment.");
                        $order->payment_complete();

                        echo($osb_response->getOutputForPlatform('payment_ok'));
                        break;
                    default:
                        $this->log("Order #$order_id is in a pending status and payment failed. Cancel order.");

                        // Add order note
                        $this->osb_add_order_note($osb_response, $order);

                        $order->update_status('failed');
                        echo($osb_response->getOutputForPlatform('payment_ko'));
                        break;
                }

                $this->log('IPN URL PROCESS END');
                die();
            } elseif ($osb_response->isAcceptedPayment() && key_exists($this->get_order_property($order, 'status'), self::$success_order_statues)) {
                $status = $osb_response->isPendingPayment() ? 'pending' : 'successfull';
                $this->log("Payment $status confirmed for order #$order_id.");

                // order success registered and payment succes received
                if ($from_server) {
                    $this->log('IPN URL PROCESS END');
                    die($osb_response->getOutputForPlatform('payment_ok_already_done'));
                } else {
                    $this->log('RETURN URL PROCESS END');
                    $this->osb_redirect($this->get_return_url($order), $iframe);
                }
            } elseif (! $osb_response->isAcceptedPayment() && ($this->get_order_property($order, 'status') === 'failed' || $this->get_order_property($order, 'status') === 'cancelled')) {
                $this->log("Payment failed confirmed for order #$order_id.");

                // order failure registered and payment error received
                if ($from_server) {
                    $this->log('IPN URL PROCESS END');
                    die($osb_response->getOutputForPlatform('payment_ko_already_done'));
                } else {
                    if (! $osb_response->isCancelledPayment()) {
                        $this->add_notice(__('Your payment was not accepted. Please, try to re-order.', 'woo-osb-payment'), 'error');
                    }

                    $this->log('RETURN URL PROCESS END');
                    $this->osb_redirect($error_url, $iframe);
                }
            } else {
                $this->log("Error ! Invalid payment result received for already saved order #$order_id. Payment result : {$osb_response->getTransStatus()}, Order status : {$this->get_order_property($order, 'status')}.");

                // registered order status not match payment result
                if ($from_server) {
                    $this->log('IPN URL PROCESS END');
                    die($osb_response->getOutputForPlatform('payment_ko_on_order_ok'));
                } else {
                    // fatal error, empty cart
                    $woocommerce->cart->empty_cart();
                    $this->add_notice(__('An error has occured in the payment process.', 'woo-osb-payment'), 'error');

                    $this->log('RETURN URL PROCESS END');
                    $this->osb_redirect($cart_url, $iframe);
                }
            }
        }
    }

    private function is_new_order($order, $trs_id)
    {
        if ($this->get_order_property($order, 'status') === 'pending') {
            return true;
        }

        if ($this->get_order_property($order, 'status') === 'failed'
            || $this->get_order_property($order, 'status') === 'cancelled') {
            return get_post_meta((int) $this->get_order_property($order, 'id'), 'Transaction ID', true) !== $trs_id;
        }

        return false;
    }

    public function osb_complete_order_status($status, $order_id)
    {
        $order = new WC_Order((int)$order_id);

        if ((strpos($this->get_order_property($order, 'payment_method'), 'osb') === 0)
            && ($this->get_general_option('order_status_on_success') != 'default')) {
            return $this->get_general_option('order_status_on_success');
        }

        return  $status;
    }

    protected function log($msg)
    {
        if (! $this->debug) {
            return;
        }

        $this->logger->add('osb', $msg);
    }

    protected function clear_notices()
    {
        global $woocommerce;

        if (function_exists('wc_clear_notices')) {
            wc_clear_notices();
        } else {
            $woocommerce->clear_messages();
        }
    }

    protected function add_notice($msg, $type = 'success')
    {
        global $woocommerce;

        if (function_exists('wc_add_notice')) {
            wc_add_notice($msg, $type);
        } else {
            if ($type == 'error') {
                $woocommerce->add_error($msg);
            } else {
                $woocommerce->add_message($msg);
            }
        }
    }

    public function osb_add_order_email_payment_result($order, $sent_to_admin, $plain_text = false)
    {
        if (strpos($this->get_order_property($order, 'payment_method'), 'osb') !== 0) {
            return;
        }

        $trans_id = get_post_meta((int) $this->get_order_property($order, 'id'), 'Transaction ID', true);
        if (! $trans_id) {
            return;
        }

        $notes = $this->get_order_notes($this->get_order_property($order, 'id'));
        foreach ($notes as $note) {
            if (strpos($note, $trans_id) !== false) {
                $osb_order_note = $note;
                break;
            }
        }

        if (isset($osb_order_note)) {
            if ($plain_text) {
                echo strtoupper(__('Payment', 'woo-osb-payment')) . "\n\n" . $osb_order_note . "\n\n";
            } else {
                echo '<h2>' . __('Payment', 'woo-osb-payment') . '</h2><p>' . str_replace("\n", '<br>', $osb_order_note) . '</p>';
            }
        }
    }

    /**
     * Get all notes of a specified order.
     * @return array[string]
     */
    private function get_order_notes($order_id)
    {
        $exclude_fnc = class_exists('WC_Comments') ? array('WC_Comments', 'exclude_order_comments') :
            'woocommerce_exclude_order_comments';

        remove_filter('comments_clauses', $exclude_fnc, 10);

        $comments = get_comments(array(
            'post_id' => $order_id,
            'status' => 'approve',
            'type'    => 'order_note'
        ));
        $notes = wp_list_pluck($comments, 'comment_content');

        add_filter('comments_clauses', $exclude_fnc, 10, 1);

        return $notes;
    }

    protected function get_order_property($order, $property_name)
    {
        $method = 'get_' . $property_name;

        if (method_exists($order, $method)) {
            return $order->$method();
        } else {
            return isset($order->$property_name) ? $order->$property_name : null;
        }
    }

    private function osb_redirect($url, $iframe = false)
    {
        if (! $iframe) {
            wp_redirect($url);
        } else {
            echo '<div style="text-align: center;">
                      <img src="' . esc_url(WC_OSB_PLUGIN_URL . 'assets/images/loading_big.gif') . '">
                  </div>';

            echo '<script type="text/javascript">
                    var url = "'.$url.'";

                    if (window.top) {
                      window.top.location = url;
                    } else {
                      window.location = url;
                    }
                  </script>';
        }

        exit();
    }

    private function osb_add_order_note($osb_response, $order)
    {
        $note = $osb_response->getCompleteMessage("\n");

        if ($osb_response->get('brand_management')) {
            $brand_info = json_decode($osb_response->get('brand_management'));
            $msg_brand_choice = "\n";

            if (isset($brand_info->userChoice) && $brand_info->userChoice) {
                $msg_brand_choice .= __('Card brand chosen by buyer.', 'woo-osb-payment');
            } else {
                $msg_brand_choice .= __('Default card brand used.', 'woo-osb-payment');
            }

            $note .= $msg_brand_choice;
        }

        if (! $osb_response->isCancelledPayment()) {
            $note .= "\n";
            $note .= sprintf(__('Transaction : %s.', 'woo-osb-payment'), $osb_response->get('trans_id'));
        }

        $note .= "\n";
        $note .= sprintf(__('Transaction status : %s.', 'woo-osb-payment'), $osb_response->getTransStatus());

        $order->add_order_note($note);
    }
}
